"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.EventEmitter = void 0;
var utils_1 = require("./utils");
var EventNames = {
    INITIALIZED: 'initialized',
    ERROR: 'error',
    VARIABLE_UPDATED: 'variableUpdated',
    FEATURE_UPDATED: 'featureUpdated',
};
var EventEmitter = /** @class */ (function () {
    function EventEmitter() {
        this.events = {};
    }
    EventEmitter.prototype.subscribe = function (key, handler) {
        (0, utils_1.checkParamType)('key', key, 'string');
        (0, utils_1.checkParamType)('handler', handler, 'function');
        var eventNames = Object.keys(EventNames).map(function (e) { return e.toLowerCase(); });
        if (!eventNames.includes(key) &&
            !key.startsWith(EventNames.VARIABLE_UPDATED) &&
            !key.startsWith(EventNames.FEATURE_UPDATED)) {
            throw new Error('Not a valid event to subscribe to');
        }
        else if (!this.events[key]) {
            this.events[key] = [handler];
        }
        else {
            this.events[key].push(handler);
        }
    };
    EventEmitter.prototype.unsubscribe = function (key, handler) {
        (0, utils_1.checkParamType)('key', key, 'string');
        var eventNames = Object.keys(EventNames).map(function (e) { return e.toLowerCase(); });
        if (!eventNames.includes(key)) {
            return;
        }
        else if (!handler) {
            this.events[key] = [];
        }
        else {
            this.events[key] = this.events[key].filter(function (eventHandler) { return eventHandler !== handler; });
        }
    };
    EventEmitter.prototype.emit = function (key) {
        var args = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            args[_i - 1] = arguments[_i];
        }
        (0, utils_1.checkParamType)('key', key, 'string');
        var handlers = this.events[key];
        if (!handlers) {
            this.events[key] = [];
            return;
        }
        handlers.forEach(function (handler) {
            handler.apply(void 0, args);
        });
    };
    EventEmitter.prototype.emitInitialized = function (success) {
        this.emit(EventNames.INITIALIZED, success);
    };
    EventEmitter.prototype.emitError = function (error) {
        this.emit(EventNames.ERROR, error);
    };
    EventEmitter.prototype.emitVariableUpdates = function (oldVariableSet, newVariableSet, variableDefaultMap) {
        var _this = this;
        var keys = Object.keys(oldVariableSet).concat(Object.keys(newVariableSet));
        keys.forEach(function (key) {
            var oldVariableValue = oldVariableSet[key] && oldVariableSet[key].value;
            var newVariable = newVariableSet[key];
            var newVariableValue = newVariable && newVariableSet[key].value;
            if (oldVariableValue !== newVariableValue) {
                var variables = variableDefaultMap[key] && Object.values(variableDefaultMap[key]);
                if (variables) {
                    variables.forEach(function (variable) {
                        var _a;
                        variable.value = newVariableValue !== null && newVariableValue !== void 0 ? newVariableValue : variable.defaultValue;
                        (_a = variable.callback) === null || _a === void 0 ? void 0 : _a.call(variable, variable.value);
                    });
                }
                _this.emit("".concat(EventNames.VARIABLE_UPDATED, ":*"), key, newVariable);
                _this.emit("".concat(EventNames.VARIABLE_UPDATED, ":").concat(key), key, newVariable);
            }
        });
    };
    EventEmitter.prototype.emitFeatureUpdates = function (oldFeatureSet, newFeatureSet) {
        var _this = this;
        var keys = Object.keys(oldFeatureSet).concat(Object.keys(newFeatureSet));
        keys.forEach(function (key) {
            var oldFeatureVariation = oldFeatureSet[key] && oldFeatureSet[key]._variation;
            var newFeature = newFeatureSet[key];
            var newFeatureVariation = newFeature && newFeatureSet[key]._variation;
            if (oldFeatureVariation !== newFeatureVariation) {
                _this.emit("".concat(EventNames.FEATURE_UPDATED, ":*"), key, newFeature);
                _this.emit("".concat(EventNames.FEATURE_UPDATED, ":").concat(key), key, newFeature);
            }
        });
    };
    return EventEmitter;
}());
exports.EventEmitter = EventEmitter;
//# sourceMappingURL=EventEmitter.js.map